/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __draw_ViewMode_h__
#define __draw_ViewMode_h__

namespace fe
{

/**************************************************************************//**
	@brief Configuration for display

	@ingroup draw
*//***************************************************************************/
class FE_DL_EXPORT ViewMode: public Counted
{
	public:
		enum Projection
		{
			e_none,
			e_perspective,
			e_ortho,
			e_current
		};

				ViewMode(void):
					m_fovy(30.0f),
					m_nearplane(0.1f),
					m_farplane(1e4f),
					m_zoom(1.0f)
				{
					set(m_viewport,0,0,1,1);
					setIdentity(m_cameraMatrix);
					set(m_center);
					setName("ViewMode");
				}

				ViewMode(const ViewMode& rViewMode) : Counted()
				{	operator=(rViewMode);
					setName("ViewMode"); }

		ViewMode& operator=(const ViewMode& rViewMode)
				{
					m_cameraMatrix=rViewMode.m_cameraMatrix;
					m_viewport=rViewMode.m_viewport;
					m_center=rViewMode.m_center;
					m_fovy=rViewMode.m_fovy;
					m_nearplane=rViewMode.m_nearplane;
					m_farplane=rViewMode.m_farplane;
					m_zoom=rViewMode.m_zoom;

					return *this;
				}

				/// @brief Define region where to draw
		void	setViewport(const Box2i& rect)		{ m_viewport=rect; }
const	Box2i&	viewport(void) const				{ return m_viewport; }

		void	setCamera(const SpatialTransform& matrix)
				{	m_cameraMatrix=matrix; }
const SpatialTransform& camera(void)
				{	return m_cameraMatrix; }

		void	setPerspective(Real fovy,Real nearplane,Real farplane)
				{	m_fovy=fovy;
					m_nearplane=nearplane;
					m_farplane=farplane; }
		void	getPerspective(Real& fovy,Real& nearplane,Real& farplane)
				{	fovy=m_fovy;
					nearplane=m_nearplane;
					farplane=m_farplane; }

		void	setOrtho(Real a_zoom, const Vector2 &a_center)
				{	m_zoom = a_zoom;
					m_center = a_center; }
		void	getOrtho(Real &a_zoom, Vector2 &a_center) const
				{	a_zoom = m_zoom;
					a_center = m_center; }

	private:

		SpatialTransform	m_cameraMatrix;
		Box2i				m_viewport;
		Vector2				m_center;
		Real				m_fovy;
		Real				m_nearplane;
		Real				m_farplane;
		Real				m_zoom;
};

inline String print(const sp<ViewMode> spViewMode)
{
	Real fovy;
	Real nearplane;
	Real farplane;
	spViewMode->getPerspective(fovy,nearplane,farplane);

	Real zoom;
	Vector2 center;
	spViewMode->getOrtho(zoom,center);

	const SpatialTransform xform=spViewMode->camera();

	String s;
	s.sPrintf("viewport %s fovy %g near %g far %g"
			" zoom %g center %s camera:\n%s",
			c_print(spViewMode->viewport()),
			fovy,nearplane,farplane,
			zoom,c_print(center),c_print(xform));
	return s;
}

} // namespace

#endif /* __draw_ViewMode_h__ */
