/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "json/json.pmh"

#define FE_SACG_DEBUG	FALSE

namespace fe
{

sp<SurfaceAccessorI> SurfaceAccessibleGeo::accessor(
	Element a_element,String a_name,Creation a_create)
{
	sp<SurfaceAccessorGeo> spAccessor(new SurfaceAccessorGeo);

	if(spAccessor.isValid())
	{
		spAccessor->setJsonRoot(m_spJsonRoot);
		if(spAccessor->bind(a_element,a_name) ||
				a_create==SurfaceAccessibleI::e_createMissing)
		{
			spAccessor->setSurfaceAccessible(sp<SurfaceAccessibleI>(this));
			return spAccessor;
		}
	}

	return sp<SurfaceAccessorI>(NULL);
}

sp<SurfaceAccessorI> SurfaceAccessibleGeo::accessor(
	Element a_element,Attribute a_attribute,Creation a_create)
{
	sp<SurfaceAccessorGeo> spAccessor(new SurfaceAccessorGeo);

	if(spAccessor.isValid())
	{
		spAccessor->setJsonRoot(m_spJsonRoot);
		if(spAccessor->bind(a_element,a_attribute) ||
				a_create==SurfaceAccessibleI::e_createMissing)
		{
			spAccessor->setSurfaceAccessible(sp<SurfaceAccessibleI>(this));
			return spAccessor;
		}
	}

	return sp<SurfaceAccessorI>(NULL);
}

BWORD SurfaceAccessibleGeo::load(String a_filename)
{
#if FE_SACG_DEBUG
	feLog("SurfaceAccessibleGeo::load \"%s\"\n",a_filename.c_str());
#endif

	if(m_spJsonRoot.isNull())
	{
		m_spJsonRoot=sp<JsonValue>(new JsonValue());
	}
	m_spJsonRoot->value().clear();

	std::ifstream in(a_filename.c_str());

	Json::Reader reader;
	const BWORD success=reader.parse(in,m_spJsonRoot->value());

	if(!success)
	{
		feLog("SurfaceAccessibleGeo::load parsing failed: %s\n",
				reader.getFormattedErrorMessages().c_str());
	}

	return success;
}

BWORD SurfaceAccessibleGeo::save(String a_filename)
{
#if FE_SACG_DEBUG
	feLog("SurfaceAccessibleGeo::save \"%s\"\n",a_filename.c_str());
#endif

	if(m_spJsonRoot.isNull())
	{
		return FALSE;
	}

	std::ofstream out(a_filename.c_str());

	Json::StyledWriter writer;
	out << writer.write(m_spJsonRoot->value());

	return TRUE;
}

Protectable* SurfaceAccessibleGeo::clone(Protectable* pInstance)
{
#if FE_SACG_DEBUG
	feLog("SurfaceAccessibleGeo::clone\n");
#endif

	SurfaceAccessibleGeo* pSurfaceAccessibleGeo= pInstance?
			dynamic_cast<SurfaceAccessibleGeo*>(pInstance):
			new SurfaceAccessibleGeo();

	sp<JsonValue> spNewRoot=sp<JsonValue>(new JsonValue());
	if(m_spJsonRoot.isValid())
	{
		spNewRoot->value()=m_spJsonRoot->value();
	}
	pSurfaceAccessibleGeo->setJsonRoot(spNewRoot);

	return pSurfaceAccessibleGeo;
}

} // namespace
