/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "operator/operator.pmh"

namespace fe
{

void PartitionOp::initialize(void)
{
	catalog<String>("icon")="FE_alpha";

	catalog<String>("partitionAttr")="part";
	catalog<String>("partitionAttr","label")="Partition Attr";

	catalog<String>("partitionText")="part%d";
	catalog<String>("partitionText","label")="Partition Text";

	catalog< sp<Component> >("Input Surface");
}

void PartitionOp::handle(Record& a_rSignal)
{
	const String partitionAttr=catalog<String>("partitionAttr");
	const String partitionText=catalog<String>("partitionText");

	sp<SurfaceAccessibleI> spOutputAccessible;
	if(!accessOutput(spOutputAccessible,a_rSignal)) return;

	sp<SurfaceAccessorI> spOutputPartition;
	if(!access(spOutputPartition,spOutputAccessible,
			e_point,partitionAttr)) return;

	const SurfaceAccessibleI::AtomicChange atomicChange=
			SurfaceAccessibleI::e_pointsOfPrimitives;
	const String group;
	const U32 desiredCount=0;
	const BWORD checkPages=FALSE;

	sp<SurfaceRange> spSurfaceRange=spOutputAccessible->atomizeConnectivity(
			atomicChange,group,desiredCount,checkPages);

//	feLog("PartitionOp::handle SurfaceRange %s\n",
//			spSurfaceRange->dump().c_str());

	const U32 atomicCount=spSurfaceRange->atomicCount();
	for(U32 atomicIndex=0;atomicIndex<atomicCount;atomicIndex++)
	{
		String partition;
		partition.sPrintf(partitionText.c_str(),atomicIndex);

		const SurfaceRange::MultiSpan& rMultiSpan=
				spSurfaceRange->atomic(atomicIndex);

		const U32 spanCount=rMultiSpan.spanCount();
		for(U32 spanIndex=0;spanIndex<spanCount;spanIndex++)
		{
			const SurfaceRange::Span& rSpan=rMultiSpan.span(spanIndex);
			const U32 start=rSpan[0];
			const U32 end=rSpan[1];

			for(U32 pointIndex=start;pointIndex<=end;pointIndex++)
			{
				spOutputPartition->set(pointIndex,partition);
			}
		}
	}
}

} // namespace
